@@ -191,7 +191,7 @@ helpWidget view = [whamlet|
     $maybe tt <- fvTooltip view
       <span .help-block>#{tt}
     $maybe err <- fvErrors view
-      <span .help-block>#{err}
+      <span .help-block .error-block>#{err}
 |]
 
 
@@ -314,4 +314,4 @@ bootstrapSubmitId = "b:ootstrap___unique__:::::::::::::::::submit-id"
 -- >        <$> areq textField nameSettings Nothing
 -- >      where nameSettings = withAutofocus $
 -- >                           withPlaceholder "First name" $
--- >                           (bfs ("Name" :: Text))
\ No newline at end of file
+-- >                           (bfs ("Name" :: Text))
